# Generated by Django 3.0.6 on 2020-05-17 11:58
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="CASAccount",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "cas_login",
                    models.CharField(
                        max_length=1023, unique=True, verbose_name="login CAS"
                    ),
                ),
                (
                    "entrance_year",
                    models.SmallIntegerField(
                        verbose_name="année de création du compte CAS"
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="cas_account",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="utilisateurice",
                    ),
                ),
            ],
            options={
                "verbose_name": "Compte CAS",
                "verbose_name_plural": "Comptes CAS",
            },
        ),
    ]
